@@ -1,4 +1,4 @@
-# Copyright (c) 2019, Adrian Dusa
+# Copyright (c) 2020, Adrian Dusa
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -40,7 +40,7 @@ function(expression = "", data = NULL, separate = FALSE) {
             data <- get(unlist(strsplit(gsub("with\\(", "", syscalls[withdata]), split = ","))[1], envir = length(syscalls) - withdata)
         }
         else {
-            colnms <- colnames(validateNames(notilde(expression), sort(toupper(eval.parent(parse(text = "ls()", n = 1))))))
+            colnms <- colnames(validateNames(notilde(expression), sort(eval.parent(parse(text = "ls()", n = 1)))))
             data <- vector(mode = "list", length = length(colnms))
             for (i in seq(length(data))) {
                 data[[i]] <- eval.parent(parse(text = sprintf("get(\"%s\")", colnms[i]), n = 1))
@@ -59,7 +59,6 @@ function(expression = "", data = NULL, separate = FALSE) {
     pp <- lapply(pp, function(x) x[retain])
     ppm <- ppm[, retain, drop = FALSE]
     data <- data[, retain, drop = FALSE]
-    colnames(data) <- toupper(colnames(data))
     infodata <- getInfo(data)
     if (any(infodata$hastime)) {
         data <- infodata$data[, colnames(data), drop = FALSE]
